///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2008
//
// Filename: CHAM_PIC_I2C_SPI_DRV_V010.C
//
// Original Author: Joshua Hintze
//
// Last Modified: 8.31.08
//
// Description: SPI/I2C library file
//
//
//
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// include local CHAM API header files
#include "CHAM_PIC_SYSTEM_V010.h"
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// I2C FUNCTIONS
//
// Note: Some microcontrollers call the I2C interface, TWI, for two wire interface. Typically this just means it has
// a little bit more features.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// I/O pin map for I2C and SPI peripherals
//
// SPI_CSn  | RB10 | Pin 21  | Output
// SPI_MOSI | RB11 | Pin 22  | Output
// SPI_MISO | RB12 | Pin 23  | Input
// SPI_SCLK | RB13 | Pin 24  | Output
//
// SCL      | RB8 | Pin 17 | Output
// SDA      | RB9 | Pin 18 | Input/Output (controlled by peripheral once enabled)
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void I2C_Init(unsigned long ClockHz)
{
	// set up I2C interface and I/O ports, default I2C_DEFAULT_CLOCK is 100K HZ in header

	// If no baud rate is set, setup baud rate to about 100KHz
	if(ClockHz == 0) ClockHz = I2C_DEFAULT_CLOCK;

	// Baud rate formula is as follows
	//	            [    Fcy			    Fcy      ]
	//	I2CxBRG =   [	------		-     -------    ]
	//				[ Clock Rate        10,000,000   ]
	I2C1BRG = ((g_FCY/ClockHz) - (g_FCY/10e6)) - 1;

	// Enable the I2C peripheral
	I2C1CONbits.I2CEN = 1;

} // end I2C_Init




void I2C_WaitTillIdle()
{
    /// Wait until I2C Bus is Inactive
    while(I2C1CONbits.SEN || I2C1CONbits.PEN || I2C1CONbits.RCEN ||
          I2C1CONbits.RSEN || I2C1CONbits.ACKEN || I2C1STATbits.TRSTAT);
}





///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SPI FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void SPI_Init(void)
{
	// initializes dedicated SPI hardware pins

	UNLOCK_IO_REMAP;			// Required that we unlock the peripheral pin repmap registers
	RPINR20bits.SDI1R = 12; 	// Map to RB12 for MISO
	AD1PCFGLbits.PCFG12 = 1;	// Set RB12 pin to a digital input and not analog
	RPOR6bits.RP13R = 0b1000;	// Map RB13 to SCLK (See datasheet table 10-3)
	RPOR5bits.RP11R = 0b0111;	// Map RB11 to MOSI
	LOCK_IO_REMAP;

	// CS to output, the other pins will automatically be set to the correct direction when we enable the peripheral
	TRISBbits.TRISB15 = 0;		// Two bits used to select decoder
	TRISBbits.TRISB14 = 0;
	
	// set CS to HIGH, disable
	SPI_SET_CS(SPI_CS_1);		// We'll use SPI_CS_1 as the OFF

	// This enables the spi to one of the most common configurations, commonly referred to SPI Mode 0
	// SPI Polarity = 0
	// SPI Phase = 0
	// Master enabled
	// Prescalar = FCY/64 = about 0.625 Mbps

	// Set Master mode
	SPI1CON1bits.MSTEN = 1;

	// Set it so serial output changes on transition of active to idle (falling edge)
	SPI1CON1bits.CKE = 1;

	// Primary prescalar 64:1
	SPI1CON1bits.PPRE = 0x01;

	// Secondary prescalar 1:1
	SPI1CON1bits.SPRE = 0x05;

	// Now enable the SPI peripheral
	SPI1STATbits.SPIEN = 1;

} // end SPI_Init





void SPI_Write(unsigned char Data8)
{
	// Wait until flag is cleared (meaning we can send more data)
	while(SPI1STATbits.SPITBF)

	// this function writes a Data8 to the SPI transmitter
	SPI1BUF = Data8;

} // end SPI_Write





unsigned char SPI_Read(unsigned char *Success)
{
	// this function reads a value from the SPI receiver and returns it and sets the success value if one is
	// provided.

    // Check for Receive buffer full status bit of status register
    if (SPI1STATbits.SPIRBF)
    {
		// Reset the receive overflow flag just incase
        SPI1STATbits.SPIROV = 0;

		// success
		if(Success)
			*Success = 1;

        return (SPI1BUF);   // return byte read

    }

	// RBF bit is not set return error
	if(Success)
		*Success = 0;

	return 0;
} // end SPI_Read





unsigned char SPI_WriteRead(unsigned char Data8)
{
	// this functions writes Data8 to the SPI interace while reading the next 8-bits from the slave and returning
	// the byte

	// Clear out any previous receive overflow errors so we can actually receive new data
	SPI1STATbits.SPIROV = 0;

	// send the data out
	SPI1BUF = Data8;

	// wait for the data to transmit and for the receiver to shift in results
	while(SPI1STATbits.SPITBF);

	// wait for new data to arrive
	while(!SPI1STATbits.SPIRBF);

	// return value from SPI interface
	return(SPI1BUF);

} // end SPI_WriteRead




int SPI_Prop_Print_String(int Device, char *String)
{
	// print the string to Prop over SPI channel
	
	static int   cmd    = 0, 
	             data   = 0, 
	             status = 0;
	
	if (Device == DEVICE_NTSC)
	   cmd = GFX_CMD_NTSC_PRINTCHAR;
	else
	   cmd = GFX_CMD_VGA_PRINTCHAR;
	
	// print the string until NULL reached
	while ((data = *String++) != NULL)
	{
	    status = SPI_Prop_Send_Cmd( cmd, data, status);
	    //DELAY_US( SPI_PROP_STRING_DELAY_US ); 
	}
	
	// return last status value
	return ( status );

} // end SPI_Prop_Print_String



/*
long SPI_Prop_Send_Cmd(unsigned char Command, unsigned char Data, unsigned char Status)
{
	// Sends out a propeller command with given arguments via the SPI port and returns
	// the current status code which is relevant for some commands

	long status_low, status_mid, status_high;

	// Select the propeller
	SPI_SET_CS(SPI_CS_PROPELLER);

	// Send the command code
	status_low = SPI_WriteRead(Command);
	DELAY_US(SPI_PROP_CMD_DELAY_US);

	// Send the first byte
	status_mid = SPI_WriteRead(Data);
	DELAY_US(SPI_PROP_CMD_DELAY_US);

	// Send the second byte
	status_high = SPI_WriteRead(Status);
	DELAY_US(SPI_PROP_CMD_DELAY_US);

	// Deselect the propeller (CH0 = Null)
	SPI_SET_CS(SPI_CS_0);
	DELAY_US(SPI_PROP_CMD_DELAY_US);

	// return status
	// note: ignore high byte for now, so we don't have to deal with sign issues, right now, the Prop driver sends back data in 24-bit format
	// but we ignore the upper 8-bits, thus the return values have a range of +-32768 and a total mag of 65535, however, signs are correct
	// but, if we go to 24-bit data then a negative 24-bit number when casted to a long will be incorrect, since there are no 24-bit data
	// types in C, thus we would have to test the sign bit and do a 24->31 bit sign extension, blah -- so for now, 16-bit signed/unsigned return
	// values. Bottom line, Prop driver ALWAYS returns all 24-bits, but all returns in the driver are 8/16 bit, so its not going to break anything
	// but the ability is there if you need 3-bytes back each time.
	//
	// if you want to add 24-bit returns, then uncomment the line below
	//return( (status_mid << 16) | (status_mid << 8) | (status_low) ); // 24-bit return, only + works right, negatives will have trouble
	return( (status_mid << 8) | (status_low) ); // 16-bit return, signs work out
}*/


// this function is used to send SPI commands to the Propeller slave processor
long SPI_Prop_Send_Cmd2(unsigned char cmd, unsigned char data, unsigned char status)
{
    long status_low, status_high, status_busy;
 
    // set CS to SPI select, select Prop SPI channel = 2
    SPI_SET_CS(SPI_CS_PROPELLER);
 
    // send command byte and retrieve low byte of result
    status_low = SPI_WriteRead( cmd );                       
    DELAY_US(SPI_PROP_CMD_DELAY_US);
 
    // send command byte and retrieve high byte of result                    
    status_high = SPI_WriteRead( data );                       
    DELAY_US(SPI_PROP_CMD_DELAY_US);
   
    // send command byte and retrieve busy byte of result                    
    status_busy = SPI_WriteRead( status );
    DELAY_US(SPI_PROP_CMD_DELAY_US);
 
    // set CS to SPI select channel 0 (null)
    SPI_SET_CS(SPI_CS_0);
    DELAY_US(SPI_PROP_CMD_DELAY_US);
 
    return( (status_busy << 16) | (status_high << 8) | (status_low) );
 
} // end SPI_Prop_Send_Cmd2

// this function waits for any previous command to complete and then initiates a new command
long SPI_Prop_Send_Cmd(unsigned char cmd, unsigned char data, unsigned char status)
{
    // wait for previous command to complete
    while (SPI_Prop_Send_Cmd2(CMD_NULL, 0, 0) & 0x00ff0000)
        ;
 
    // initiate the new command
    return SPI_Prop_Send_Cmd2(cmd, data, status) & 0x0000ffff;
 
} // end SPI_Prop_Send_Cmd


