///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2008
//
// Filename: CHAM_PIC_SYSTEM_V010.H
//
// Original Author: Joshua Hintze
//
// Last Modified: 8.31.08
//
// Description: Main system header file, it includes all our common types, macros, functions that are used
//				throughout the Chameleon library.
//
//
//
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// We'll include the chip header file here just incase we forget it somewhere
#include "p24HJ128GP502.h"

// Include libpic30.h for some useful functions (flash copy, uart simulation files, etc.)
// Most important is the prototype for the __delay32 assembly program automatically linked
#include "libpic30.h"

// watch for multiple inclusions
#ifndef CHAM_PIC_SYSTEM
#define CHAM_PIC_SYSTEM

// support C++ compilers
 #ifdef __cplusplus
 extern "C" {
 #endif 
 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// A couple simple delay routines
#define CYCLES_PER_SEC ((unsigned long)g_FCY)

// Instruction cycles per millisecond
#define CYCLES_PER_MS ((unsigned long)(g_FCY * 0.001))

// Instruction cycles per microsecond
#define CYCLES_PER_US ((unsigned long)(g_FCY * 0.000001))

//__delay32 is provided by the compiler, delay some # seconds
#define DELAY_SEC(sec)  __delay32(CYCLES_PER_SEC * ((unsigned long) sec))

//__delay32 is provided by the compiler, delay some # of milliseconds
#define DELAY_MS(ms)  __delay32(CYCLES_PER_MS * ((unsigned long) ms))

// Delay some number of microseconds
#define DELAY_US(us)  __delay32(CYCLES_PER_US * ((unsigned long) us))

// Delay some clocks, minimum is 12 clocks
#define DELAY_CLOCKS(clocks) __delay32(clocks)

// Cause a one cycle stall
#define NOP {__asm__ volatile ("nop");}

// Used to unlock the IO remapable pin ability
#define UNLOCK_IO_REMAP {__asm__ volatile ( "MOV #OSCCON, w1 \n" \
				   						"MOV #0x46, w2 \n" \
				   						"MOV #0x57, w3 \n" \
				   						"MOV.b w2, [w1] \n" \
				   						"MOV.b w3, [w1] \n" \
				   						"BCLR OSCCON,#6"); }

// Used to lock the IO remapable pin ability
#define LOCK_IO_REMAP {__asm__ volatile (  "MOV #OSCCON, w1 \n" \
									   "MOV #0x46, w2 \n" \
									   "MOV #0x57, w3 \n" \
									   "MOV.b w2, [w1] \n" \
									   "MOV.b w3, [w1] \n" \
									   "BSET OSCCON, #6" ); }


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Max chip specified instruction rate (40 MIPS)
#define MAX_FCY_RATE 40000000L

// Pin direction constants. PICs use 0 for output, 1 for input on TRIS registers
#define PIN_DIR_OUTPUT	0
#define PIN_DIR_INPUT	1

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// The choosen operating clock rate
extern unsigned long g_FCY;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// generic input device data record type, helps get back data from various multidata calls
typedef struct gid_event_type
    {

    int x,y,z;    // position data
    int buttons;  // buttons, bit encoded
                  // bit4 = right-side button
                  // bit3 = left-side button
                  // bit2 = center/scrollwheel button
                  // bit1 = right button
                  // bit0 = left button

    } gid_event, *gid_event_ptr;

// Configures the clock PLL multiplier, pass in FCY
void SYS_ConfigureClock(unsigned long FCY);


// watch for multiple inclusions
#endif



 // end support for C++ compilers
 #ifdef __cplusplus
 }
 #endif 


